#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <glib.h>

#include <g3dviewer_plugin.h>

int plugin_load(const char *filename, G3DImage *image)
{
  FILE *f;
  unsigned int filesize, offset, headsize, compression;
  int x, y, c;

  f = fopen(filename, "r");
  if(f == NULL)
  {
    g_printerr("couldn't open image file '%s': %s\n", filename,
               strerror(errno));
    return EXIT_FAILURE;
  }

  /* bitmap file always starts with 'BM' */
  if(g3d_read_int16_le(f) != ('B' | ('M' << 8)))
  {
    g_printerr("bitmap magic not found: image seems to be corrupt\n");
    return EXIT_FAILURE;
  }

  image->name = g_strdup(filename);

  filesize = (unsigned int)g3d_read_int32_le(f);      /* file size */
  g3d_read_int32_le(f);                               /* 2 x UINT16 reserved */
  offset   = (unsigned int)g3d_read_int32_le(f);      /* offset of data */
  headsize = (unsigned int)g3d_read_int32_le(f);      /* size of header */
  image->width  = (unsigned int)g3d_read_int32_le(f); /* width */
  image->height = (unsigned int)g3d_read_int32_le(f); /* height */
  g3d_read_int16_le(f);                               /* num of color planes */
  image->depth  = (unsigned int)g3d_read_int16_le(f); /* bits per pixel */
  compression   = (unsigned int)g3d_read_int32_le(f); /* compression */
  g3d_read_int32_le(f);                               /* image size */
  g3d_read_int32_le(f);                               /* v/res (dpi) */
  g3d_read_int32_le(f);                               /* h/res (dpi) */

  fseek(f, offset, SEEK_SET);

  image->pixeldata = g_malloc0(image->width*image->height*(image->depth/8));
  for(y=image->height-1; y>=0; y--)
  {
#if DEBUG > 5
    g_printerr("| ");
#endif
    for(x=0; x<image->width; x++)
    {
      switch(image->depth)
      {
        case 8:
          image->pixeldata[y*image->width+x] = g3d_read_int8(f);
#if DEBUG > 5
          g_printerr("%2.2X ", image->pixeldata[y*image->width+x]);
#endif
          break;
        default:
          break;
      } 
    }
#if DEBUG > 5
    g_printerr("\n");
#endif
  }
  fseek(f, offset, SEEK_SET);
  
  return EXIT_SUCCESS;
  
}

char *plugin_description(void)
{ 
  return g_strdup(
    "Plugin to read Windows Bitmap (.bmp) images\n"
    "(for use as textures or height fields)\n"
    "Author: Markus Dahms");
}

char **plugin_extensions(void)
{
  return g_strsplit("bmp", ":", 0);
}

