#include <stdlib.h>

#include <gtk/gtk.h>
#include <gtkgl/gtkglarea.h>
#include <GL/gl.h>

#include <g3dviewer_plugin.h>

#include "gl.h"
#include "glarea.h"
#include "trackball.h"

gint glarea_expose(GtkWidget *widget, GdkEventExpose *event)
{
  GtkGLArea *glarea = GTK_GL_AREA(widget);
  G3DModel *model = (G3DModel*)gtk_object_get_data(GTK_OBJECT(glarea), 
                                                   "model");
  if(event->count > 0) return TRUE;

  if(gtk_gl_area_make_current(glarea)) 
  {
    gl_draw(model);
    gtk_gl_area_swapbuffers(glarea);
  }

  return TRUE;
}

gint glarea_configure(GtkWidget *widget, GdkEventConfigure *event)
{
  if(gtk_gl_area_make_current(GTK_GL_AREA(widget)))
  {
    G3DModel *model = (G3DModel*)gtk_object_get_data(GTK_OBJECT(widget),
                                                     "model");
    glViewport(0,0, widget->allocation.width, widget->allocation.height);
    model->aspect = (float)widget->allocation.width / 
                    (float)widget->allocation.height;
#if DEBUG > 3
    g_printerr("DEBUG: glarea_configure (%f)\n", model->aspect);
#endif
  }
  return TRUE;
}

gint glarea_destroy(GtkWidget *widget)
{

  G3DModel *model = (G3DModel*)gtk_object_get_data(GTK_OBJECT(widget), 
                                                   "model");
  if(model)
  {
    g_free(model);
  }
  return TRUE;
}

gint glarea_button_pressed(GtkWidget *widget, GdkEventButton *event)
{
  G3DModel *model = (G3DModel*)gtk_object_get_data(GTK_OBJECT(widget),
                                                   "model");
  if(event->button == 1)
  {
    model->mouse.beginx = event->x;
    model->mouse.beginy = event->y;
    return TRUE;
  }
  else if(event->button == 3)
  {
    if(event->type == GDK_BUTTON_PRESS)
    {
      GtkWidget *menu = (GtkWidget*)gtk_object_get_data(GTK_OBJECT(widget),
                                                        "menu");
      gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
                     event->button, event->time);
      return TRUE;
    }
  }
  return FALSE;
}

gint glarea_motion_notify(GtkWidget *widget, GdkEventMotion *event)
{
  int x, y;
  GdkRectangle area;
  GdkModifierType state;
  G3DModel *model = (G3DModel*)gtk_object_get_data(GTK_OBJECT(widget),
                                                    "model");

  if(event->is_hint) gdk_window_get_pointer(event->window, &x, &y, &state);
  else
  {
    x = event->x;
    y = event->y;
    state = event->state;
  }

  area.x = 0;
  area.y = 0;
  area.width = widget->allocation.width;
  area.height = widget->allocation.height;

  if(state & GDK_BUTTON1_MASK)
  {
    float spin_quat[4];
    trackball(spin_quat,
      (2.0*model->mouse.beginx -              area.width) / area.width,
      (            area.height - 2.0*model->mouse.beginy) / area.height,
      (                  2.0*x -              area.width) / area.width,
      (            area.height -                   2.0*y) / area.height);
    add_quats(spin_quat, model->quat, model->quat);
    gtk_widget_draw(widget, &area);
  }

  if(state & GDK_BUTTON2_MASK)
  {
#if 0
    g_printerr("zooming... (%f)\n", model->zoom);
#endif
    model->zoom += ((y - model->mouse.beginy) / (float)area.height) * 40;
    if(model->zoom < 1)   model->zoom = 1;
    if(model->zoom > 120) model->zoom = 120;
    gtk_widget_draw(widget, &area);
  }
  model->mouse.beginx = x;
  model->mouse.beginy = y;

  return TRUE;
}

